/**
 * Created by hubert on 8/06/17.
 */
import { OzoneClient } from 'ozone-typescript-client';
import { Item, UUID, FromOzone } from 'ozone-type';
import { SearchQuery } from 'ozone-search-helper';
import SearchIterator = OzoneClient.SearchIterator;
import SearchResults = OzoneClient.SearchResults;
/**
 * Function decorator decorator to be used to wait until
 * all other decorated function resolve.
 * This decorator is aimed to be used in class that implement StatefulOzone.
 * It's purpose is to wait others ozone call finish, before sending a next one.
 *
 */
export declare function lockRequest(): (target: StatefulOzone, propertyKey: string, descriptor: PropertyDescriptor) => void;
export interface StatefulOzone {
    _currentRequest: Promise<any>;
}
/**
 * `ozone-api-item` is low level es6 module to ozone api.
 * It provide CRUD operation and search in a given collection.
 *
 * * Example
 * ```javaScript
 * const ozoneApiSearch = new OzoneApiItem(); // return instance of OzoneApiItem located in the dom
 * const result = ozoneApiSearch.on('item').getOne('an-id');
 * ```
 *
 */
export declare class OzoneApiItem<T = Item> {
    /**
     * type of the ozone collection.
     * Default value is 'item'
     */
    private collection;
    /**
     * set collection and return this to be chain by a query.
     * @param {string} collection
     * @return {OzoneApiItem} this
     */
    on(collection: string): this;
    /**
     * Set ozone collection to query
     * @param {string} collection
     */
    setCollection(collection: string): void;
    /**
     * Create or update a collection item.
     * @param data Item item to create.
     * @return {Promise<Item>}
     */
    create(data: Partial<T>): Promise<FromOzone<T> | null>;
    /**
     * Create or update a collection item.
     * @param data Item item to update.
     * @return {Promise<Item>}
     */
    update(data: Partial<T>): Promise<FromOzone<T> | null>;
    /**
     * get one collection item by uuid.
     * @param id
     * @return {Promise<Item | null>}
     */
    getOne(id: UUID): Promise<FromOzone<T> | null>;
    /**
     * delete one collection item by uuid.
     * @param id
     * @return {Promise<any>}
     */
    deleteOne(id: UUID): Promise<UUID | null>;
    /**
     * get collection items from a list of id.
     * @param ids {Array<UUID>} array of id to get
     * @return {Promise<Iterator<Item>>} promise resole with an iterator of collection item
     */
    bulkGet(ids: Array<UUID>): Promise<Array<FromOzone<T>> | null>;
    /**
     * delete items from a list of id.
     * @param ids
     * @return {Promise<Array<UUID>>} promise resole with an array of deleted id
     */
    bulkDelete(ids: Array<UUID | undefined>): Promise<Array<UUID> | null>;
    /**
     * save an array of items
     * @param items
     * @return {Promise<Iterator<Item>>} promise resole with an iterator of collection item
     */
    bulkSave(items: Array<Partial<T>>): Promise<Array<FromOzone<T>> | null>;
    /**
     * Submit ozone search query
     */
    search(search: SearchQuery): Promise<SearchGenerator<T>>;
    queryDelete(search: SearchQuery): Promise<UUID[]>;
}
/**
 * Class helper to iterate on search result.
 * * Example:
 * ```javaScript
 *   let searchQuery = new SearchQuery();
 *   searchQuery.quicksearch('');
 *   const searchGenerator = ozoneItemApi.search(searchQuery);
 *   searchGenerator.next().then((searchResult)=>{
 *               searchResult.results.forEach((item)=>{
 *                   this.push('items', item);
 *               })
 *           });
 * ```
 */
export declare class SearchGenerator<T extends Item = Item> implements StatefulOzone {
    private searchIterator;
    _currentRequest: Promise<any>;
    constructor(searchIterator: SearchIterator<T>);
    /**
     * load next array of results
     * @return {Promise<SearchResult>}
     */
    next(): Promise<SearchResults<FromOzone<T>>>;
    cancelRequest(): void;
}
