/**
 * Created by hubert on 8/06/17.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { getDefaultClient } from 'ozone-default-client';
/**
 * Function decorator decorator to be used to wait until
 * all other decorated function resolve.
 * This decorator is aimed to be used in class that implement StatefulOzone.
 * It's purpose is to wait others ozone call finish, before sending a next one.
 *
 */
export function lockRequest() {
    return function (target, propertyKey, descriptor) {
        let originalMethod = descriptor.value;
        descriptor.value = function () {
            const self = this;
            const arg = arguments;
            self._currentRequest = self._currentRequest
                // tslint:disable-next-line:no-empty
                .catch(() => { }) // error should be handled by the user.
                .then(() => {
                return originalMethod.apply(this, arg);
            });
            return self._currentRequest;
        };
    };
}
/**
 * `ozone-api-item` is low level es6 module to ozone api.
 * It provide CRUD operation and search in a given collection.
 *
 * * Example
 * ```javaScript
 * const ozoneApiSearch = new OzoneApiItem(); // return instance of OzoneApiItem located in the dom
 * const result = ozoneApiSearch.on('item').getOne('an-id');
 * ```
 *
 */
export class OzoneApiItem {
    constructor() {
        /**
         * type of the ozone collection.
         * Default value is 'item'
         */
        this.collection = 'item';
    }
    /**
     * set collection and return this to be chain by a query.
     * @param {string} collection
     * @return {OzoneApiItem} this
     */
    on(collection) {
        this.setCollection(collection);
        return this;
    }
    /**
     * Set ozone collection to query
     * @param {string} collection
     */
    setCollection(collection) {
        this.collection = collection;
    }
    /**
     * Create or update a collection item.
     * @param data Item item to create.
     * @return {Promise<Item>}
     */
    create(data) {
        return this.update(data);
    }
    /**
     * Create or update a collection item.
     * @param data Item item to update.
     * @return {Promise<Item>}
     */
    update(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const itemClient = getDefaultClient().itemClient(this.collection);
            return itemClient.save(data);
        });
    }
    /**
     * get one collection item by uuid.
     * @param id
     * @return {Promise<Item | null>}
     */
    getOne(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const itemClient = getDefaultClient().itemClient(this.collection);
            return itemClient.findOne(id);
        });
    }
    /**
     * delete one collection item by uuid.
     * @param id
     * @return {Promise<any>}
     */
    deleteOne(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const itemClient = getDefaultClient().itemClient(this.collection);
            return itemClient.deleteById(id);
        });
    }
    /**
     * get collection items from a list of id.
     * @param ids {Array<UUID>} array of id to get
     * @return {Promise<Iterator<Item>>} promise resole with an iterator of collection item
     */
    bulkGet(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const itemClient = getDefaultClient().itemClient(this.collection);
            return itemClient.findAllByIds(ids);
        });
    }
    /**
     * delete items from a list of id.
     * @param ids
     * @return {Promise<Array<UUID>>} promise resole with an array of deleted id
     */
    bulkDelete(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const itemClient = getDefaultClient().itemClient(this.collection);
            return itemClient.deleteByIds(ids.filter(id => !!id));
        });
    }
    /**
     * save an array of items
     * @param items
     * @return {Promise<Iterator<Item>>} promise resole with an iterator of collection item
     */
    bulkSave(items) {
        return __awaiter(this, void 0, void 0, function* () {
            const itemClient = getDefaultClient().itemClient(this.collection);
            return itemClient.saveAll(items);
        });
    }
    /**
     * Submit ozone search query
     */
    search(search) {
        return __awaiter(this, void 0, void 0, function* () {
            const collection = search.collection; // collection is deprecated
            if (collection) {
                this.on(collection);
            }
            const itemClient = getDefaultClient().itemClient(this.collection);
            return new SearchGenerator(itemClient.searchGenerator(search.searchRequest));
        });
    }
    queryDelete(search) {
        return __awaiter(this, void 0, void 0, function* () {
            const collection = search.collection; // collection is deprecated
            if (collection) {
                this.on(collection);
            }
            const itemClient = getDefaultClient().itemClient(this.collection);
            if (!search.searchRequest.query) {
                throw new Error('No search query');
            }
            else {
                return itemClient.queryDelete(search.searchRequest.query);
            }
        });
    }
}
/**
 * Class helper to iterate on search result.
 * * Example:
 * ```javaScript
 *   let searchQuery = new SearchQuery();
 *   searchQuery.quicksearch('');
 *   const searchGenerator = ozoneItemApi.search(searchQuery);
 *   searchGenerator.next().then((searchResult)=>{
 *               searchResult.results.forEach((item)=>{
 *                   this.push('items', item);
 *               })
 *           });
 * ```
 */
export class SearchGenerator {
    constructor(searchIterator) {
        this.searchIterator = searchIterator;
        this._currentRequest = Promise.resolve();
    }
    /**
     * load next array of results
     * @return {Promise<SearchResult>}
     */
    next() {
        return __awaiter(this, void 0, void 0, function* () {
            const { value } = yield this.searchIterator.next();
            return value;
        });
    }
    cancelRequest() {
        this.searchIterator.cancel();
    }
}
__decorate([
    lockRequest(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], SearchGenerator.prototype, "next", null);
